/* GStreamer

 * Copyright (C) 2009- Luo Jinghua <sunmoon1997@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more
 */

#ifndef __GST_SOPCAST_SRC_H__
#define __GST_SOPCAST_SRC_H__

#include <gst/gst.h>
#include <gst/base/gstpushsrc.h>
#include <glib.h>

#include <time.h>

G_BEGIN_DECLS

#define GST_TYPE_SOPCAST_SRC \
  (gst_sopcast_src_get_type())
#define GST_SOPCAST_SRC(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_SOPCAST_SRC,GstSopcastSrc))
#define GST_SOPCAST_SRC_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST((klass), \
      GST_TYPE_SOPCAST_SRC,GstSopcastSrcClass))
#define GST_IS_SOPCAST_SRC(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_SOPCAST_SRC))
#define GST_IS_SOPCAST_SRC_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_SOPCAST_SRC))

typedef struct _GstSopcastSrc GstSopcastSrc;
typedef struct _GstSopcastSrcClass GstSopcastSrcClass;

struct _GstSopcastSrc {
  GstPushSrc element;

  gchar *location;             /* Full URI. */
  int ports[2];                /* inboud/outbound ports */
  int sock;                    /* the socket connected to server */
  pid_t helper_pid;            /* The pid of helper */
  gboolean first_request;      /* The first request */
  gint percent;                /* buffering percentage */
  time_t last_update;          /* for updating stats */
  int heatebeat_fd;

  GstFlowReturn ret;           /* Return code from callback. */
  GstBuffer **outbuf;          /* Return buffer allocated by callback. */
  gboolean interrupted;        /* Signal unlock(). */
  gboolean retry;              /* Should attempt to reconnect. */

  gboolean have_size;          /* Received and parsed Content-Length
                                  header. */
  guint64 content_size;        /* Value of Content-Length header. */
  guint64 read_position;       /* Current position. */
  gboolean seekable;           /* FALSE if the server does not support
                                  Range. */
  guint64 request_position;    /* Seek to this position. */
};

struct _GstSopcastSrcClass {
  GstPushSrcClass parent_class;
};

GType gst_sopcast_src_get_type (void);

G_END_DECLS

#endif /* __GST_SOPCAST_SRC_H__ */

